package seguros;

import seguros.excepciones.*;
import java.sql.*;
import java.util.*;

/**
 * JavaBean del Administrador, la presencia de este JavaBean en la session indicar
 * que el usuario ha sido identificado correctamente.
 */
public class Admin extends Usuario {
  
  /**
   * Cambio de password del usuario admin
   * @param oldPass Password actual.
   * @param newPass Nuevo password.
   * @throws ViejoPassIncorrecto El password actual no coincide con el indicado
   */
  public void setPassword(String oldPass, String newPass) throws SQLException, ViejoPassIncorrecto {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    ResultSet resultado;
    
    sqlLine = "UPDATE administrador SET password = PASSWORD(?) WHERE ADM_ID = ? AND password = PASSWORD(?)";      
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,newPass);
    ps.setInt(2,id);
    ps.setString(3,oldPass);
    if (ps.executeUpdate() == 0) {
      throw new ViejoPassIncorrecto();
    }
    liberarConexion(conexion);
  }
  
  /**
   * Metodo privado que consulta algun campo de la tabla de la compaia
   * @return Devuelve un string con el ID de la compaia
   */
  private String getCompaniaInfo(String campo) throws NoEncontrado,SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM compania"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        String campoResultado = resultado.getString(campo);
        liberarConexion(conexion);
        return campoResultado;
      } else {
        throw new NoEncontrado();
      }
    } catch (SQLException e) {
      throw new NoEncontrado();
    }
    
  }
  
  /**
   * Mtodo privado que cambia campo con el valor indicado de la tabla compania
   * @param campo Campo a cambiar
   * @param valor Valor del campo
   */
  private void setCompaniaInfo(String campo, String valor) throws SQLException, NoEncontrado {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    ResultSet resultado;
    
    sqlLine = "UPDATE compania SET " + campo + " = ?";      
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,valor);
    ps.executeUpdate();
    liberarConexion(conexion);
  }
  
  /**
   * Consulta el ID de la compaia
   * @return Devuelve un string con el ID de la compaia
   */
  public String getCompaniaID() throws NoEncontrado,SQLException {
    return getCompaniaInfo("CO_ID");
  }
  
  /**
   * Cambia el id de la compaia y modifica todos los IDs de las polizas ya que deben ser id_compaia + id_poliza
   * @param id Nuevo ID de la compaia
   */
  public void setCompaniaID(int id) throws SQLException, NoEncontrado {
    String idActual = getCompaniaID();
    try{
      // Como es una actualizacion costosa, solo realizarla si el nuevo id es diferente del actual
      if (Integer.parseInt(idActual) != id) {
        String sqlLine;
        Connection conexion = obtenerConexion();
        Statement sentencia = conexion.createStatement();
        PreparedStatement ps;
        ResultSet resultado;
        
        sqlLine = "UPDATE compania SET CO_ID = ?";      
        ps = conexion.prepareStatement(sqlLine);
        ps.setInt(1,id);
        ps.executeUpdate();
        
        // Cambiamos los IDs de todas las polizas pq los 3 primeros digitos deben corresponder al ID de la compaia
        int idPoliza;
        String idPolizaCompleta;
        String oldIdPoliza;
        sqlLine = "SELECT PO_ID FROM polizas ORDER BY PO_ID"; 
        resultado = sentencia.executeQuery(sqlLine);
        while (resultado.next()){
          // ID actual
          oldIdPoliza = resultado.getString("PO_ID");
          // Eliminamos el codigo de compaia y obtenemos el numero real de poliza
          idPoliza = Integer.parseInt((resultado.getString("PO_ID")).substring(3));
    
          // Obtenemos el nuevo id de la poliza con el nuveo id de la compaia
          idPolizaCompleta = id+""+idPoliza;
          sqlLine = "UPDATE polizas SET PO_ID = ? WHERE PO_ID = ?";
          ps = conexion.prepareStatement(sqlLine);
          ps.setString(1,idPolizaCompleta);
          ps.setString(2,oldIdPoliza);
          ps.executeUpdate();
          
          sqlLine = "UPDATE partes_internos SET PO_ID = ? WHERE PO_ID = ?";
          ps = conexion.prepareStatement(sqlLine);
          ps.setString(1,idPolizaCompleta);
          ps.setString(2,oldIdPoliza);
          ps.executeUpdate();
          
          sqlLine = "UPDATE partes_internos SET poliza_causante = ? WHERE poliza_causante = ?";
          ps = conexion.prepareStatement(sqlLine);
          ps.setString(1,idPolizaCompleta);
          ps.setString(2,oldIdPoliza);
          ps.executeUpdate();
          
          sqlLine = "UPDATE partes_externos SET PO_ID = ? WHERE PO_ID = ?";
          ps = conexion.prepareStatement(sqlLine);
          ps.setString(1,idPolizaCompleta);
          ps.setString(2,oldIdPoliza);
          ps.executeUpdate();
        }
    
        liberarConexion(conexion);
      }
    } catch (NumberFormatException e){
    }
  }
  
  /**
   * Consulta el nombre de la compaia
   * @return Devuelve un string con el nombre de la compaia
   */
  public String getCompaniaNombre() throws NoEncontrado,SQLException {
    return getCompaniaInfo("Nombre");
  }
  
  /**
   * Modifica el nombre de la compaia
   * @param nombre Nuevo nombre de la compaia
   */
  public void setCompaniaNombre(String nombre) throws NoEncontrado,SQLException {
    setCompaniaInfo("Nombre", nombre);
  }
  
  /**
   * Consulta la URL de la web de la compaia
   * @return Devuelve un string con la URL de la compaia
   */
  public String getCompaniaURL() throws NoEncontrado,SQLException {
    return getCompaniaInfo("URLweb");
  }
  
  /**
   * Modifica la URL de la web de la compaia
   * @param url Nueva URL de la compaia
   */
  public void setCompaniaURL(String url) throws NoEncontrado,SQLException {
    setCompaniaInfo("URLweb", url);
  }
  
  /**
   * Consulta la URL del icono de la compaia
   * @return Devuelve un string con la URL del icono de la compaia
   */
  public String getCompaniaIconURL() throws NoEncontrado,SQLException {
    return getCompaniaInfo("URLicono");
  }
  
  /**
   * Modifica la URL del icono de la compaia
   * @param url Nueva URL del icono de la compaia
   */
  public void setCompaniaIconURL(String url) throws NoEncontrado,SQLException {
    setCompaniaInfo("URLicono", url);
  }

  /**
   * Consulta la URL del proxy de la compaia (ServLet encargado de la comunicacion entre compaias)
   * @return Devuelve un string con la URL del proxy de la compaia
   */
  public String getCompaniaProxyURL() throws NoEncontrado,SQLException {
    return getCompaniaInfo("URLproxy");
  }
  
  /**
   * Modifica la URL del proxy de la compaia (ServLet encargado de la comunicacion entre compaias)
   * @param url Nueva URL del proxy de la compaia
   */
  public void setCompaniaProxyURL(String url) throws NoEncontrado,SQLException {
    setCompaniaInfo("URLproxy", url);
  }

  /**
   * Consulta la URL del servidor central Homer
   * @return Devuelve un string con la URL del servidor central Homer
   */
  public String getCompaniaHomer() throws NoEncontrado,SQLException {
    return getCompaniaInfo("Homer");
  }
  
  /**
   * Modifica la URL del servidor central Homer
   * @param url Nueva URL del servidor central Homer
   */
  public void setCompaniaHomer(String url) throws NoEncontrado,SQLException {
    setCompaniaInfo("Homer", url);
  }
  
  /**
   * Consulta la URL del dtd para validar los XML que enviemos
   * @return URL del dtd
   */
  public String getCompaniaDTD() throws NoEncontrado,SQLException {
    return getCompaniaInfo("DTD");
  }
  
  /**
   * Modifica la URL del dtd para validar los XML que enviemos
   * @param URL URL del dtd
   */
  public void setCompaniaDTD(String URL) throws NoEncontrado,SQLException {
    setCompaniaInfo("DTD", URL);
  }
}