package seguros;

import seguros.*;
import seguros.excepciones.*;
import java.sql.*;
import java.util.*;
import java.io.*;
import org.jdom.*; 
import java.lang.Integer;
import javax.naming.NamingException;

/**
 * JavaBean del Agente, la presencia de este JavaBean en la session indicar
 * que el usuario ha sido identificado correctamente.
 */
public class Agente extends Usuario {
  
  /**
   * Cambio de password del usuario agente
   * @param oldPass Password actual.
   * @param newPass Nuevo password.
   * @throws ViejoPassIncorrecto El password actual no coincide con el indicado
   */
  public void setPassword(String oldPass, String newPass) throws SQLException, ViejoPassIncorrecto {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    ResultSet resultado;
    
    sqlLine = "UPDATE agentes SET password = PASSWORD(?) WHERE AG_ID = ? AND password = PASSWORD(?)";      
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,newPass);
    ps.setInt(2,id);
    ps.setString(3,oldPass);
    if (ps.executeUpdate() == 0) {
      throw new ViejoPassIncorrecto();
    }
    liberarConexion(conexion);
  }
  
  /**
   * Comprueba si ja existe un administrador con ese login
   * @param login Login a buscar
   * @return Retorna si existe o no dicho login
   */
  private boolean existeLoginAdmin(String login) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM administrador WHERE login = ?"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        liberarConexion(conexion);
        return true;
      } else {
        liberarConexion(conexion);
        return false;
      }
    } catch (SQLException e) {
      return false;
    }
  }
  
  /**
   * Comprueba si ja existe un administrador con ese login
   * @param login Login a buscar
   * @return Retorna si existe o no dicho login
   */
  private boolean existeLoginAgente(String login) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM agentes WHERE login = ?"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        liberarConexion(conexion);
        return true;
      } else {
        liberarConexion(conexion);
        return false;
      }
    } catch (SQLException e) {
      return false;
    }
    
  }
  
  /**
   * Comprueba si ja existe un administrador con ese login
   * @param login Login a buscar
   * @return Retorna si existe o no dicho login
   */
  private boolean existeLoginCliente(String login) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM clientes WHERE login = ?"; 
    try {
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      resultado = ps.executeQuery();
      if (resultado.next()) {
        liberarConexion(conexion);
        return true;
      } else {
        liberarConexion(conexion);
        return false;
      }
    } catch (SQLException e) {
      return false;
    }
    
  }
  
  /**
   * Da de alta un cliente asociado a este agente
   * @param login Login
   * @param pass Password
   * @param nombre Nombre
   * @param DNI DNI
   * @param mail E-Mail
   * @throws LoginOcupado si el login no esta disponible
   * @throws FaltanDatos si no ha rellenado todos los datos
   * @throws SQLException
   */
  public void altaCliente(String login, String pass, String nombre, int telf, int DNI, String mail) throws LoginOcupado,SQLException,FaltanDatos {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    if ((existeLoginAdmin(login)) || (existeLoginAgente(login)) ||(existeLoginCliente(login))){
      throw new LoginOcupado();
    }
    if ((login.equals("")) || (pass.equals("")) || (nombre.equals("")) || (mail.equals(""))) {
      throw new FaltanDatos();
    }
    sqlLine = "INSERT INTO clientes VALUES ('',?,?,PASSWORD(?),?,?,?,?)";
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,login);
    ps.setInt(2,id);
    ps.setString(3,pass);
    ps.setString(4,nombre);
    ps.setInt(5,telf);
    ps.setInt(6,DNI);
    ps.setString(7,mail);
    ps.executeUpdate();
    
    liberarConexion(conexion);
  }
  
  
  
  /**
   * Lista los clientes de este agente
   * @return Coleccin Vector que contiene todos los clientes en forma de FichaUsuario
   */
  public Collection listarClientes() throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    Collection c = new Vector();
    FichaUsuario f;
    sqlLine = "SELECT * FROM clientes WHERE AG_ID = ? ORDER BY nombre";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,id);
    resultado = ps.executeQuery();
    while (resultado.next()) {
      // Creamos la FichaUsuario
      f = new FichaUsuario(resultado.getInt("CL_ID"), resultado.getString("nombre"),
              resultado.getInt("telf"), resultado.getInt("DNI"), resultado.getString("mail"),
              resultado.getString("login"));
      // Aadimos la Poliza al vector
      c.add(f);
    }
    
    liberarConexion(conexion);
    return c;
  }
  
  /**
   * Comprueba si el cliente especificado es de este agente
   * @param clienteID Codigo del cliente
   * @return true si pertenece a este agente; false sino
   */
  public boolean esClienteMio(String poliza) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT * FROM polizas WHERE PO_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,poliza);
    resultado = ps.executeQuery();

    if (resultado.next()) {
      String clienteID = resultado.getString("CL_ID");
      
      sqlLine = "SELECT * FROM clientes WHERE AG_ID = ? AND CL_ID = ?";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,id);
      ps.setString(2,clienteID);
      resultado = ps.executeQuery();
      
      if (resultado.next()) {
        liberarConexion(conexion);
        return true;
      } else {
        liberarConexion(conexion);
        return false;
      }
    }
    return false;
    
  }
  
  
  /**
   * Dar de baja el cliente indicado, siempre que sea de este agente
   */
  public void bajaCliente(int idCliente) throws SQLException {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "DELETE FROM clientes WHERE CL_ID= ? AND AG_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idCliente);
    ps.setInt(2,id);
    ps.executeUpdate();
    liberarConexion(conexion);
  }
  
  
  /**
   * Obtiene la informacin del cliente especificado, siempre que sea de este agente
   * @return FichaUsuario del cliente que hemos solicitado
   */
  public FichaUsuario obtenerCliente(int idCliente) throws SQLException, NoEncontrado{
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    FichaUsuario f;
    
    sqlLine = "SELECT * FROM clientes WHERE AG_ID = ? AND CL_ID = ?"; 
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,id);
    ps.setInt(2,idCliente);
    resultado = ps.executeQuery();
    if (resultado.next()) {
      f = new FichaUsuario(resultado.getInt("CL_ID"), resultado.getString("nombre"),
              resultado.getInt("telf"), resultado.getInt("DNI"), resultado.getString("mail"),
              resultado.getString("login"));
      liberarConexion(conexion);
      return f;
    } else {
      throw new NoEncontrado();
    }
  }
  
  /**
   * Modifica el cliente con el id indicado y que pertenezca a este agente
   * @param login Login
   * @param pass Password
   * @param nombre Nombre
   * @param DNI DNI
   * @param mail E-Mail
   * @throws LoginOcupado si el login no esta disponible
   * @throws FaltanDatos si no ha rellenado todos los datos
   * @throws SQLException
   */
  public void modificarCliente(int idCliente, String login, String pass, String nombre, int telf, int DNI, String mail) throws SQLException,FaltanDatos {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    if ((login.equals("")) || (nombre.equals("")) || (mail.equals(""))) {
      throw new FaltanDatos();
    }
    
    if (pass.equals("")) {
      // No actualizar pasword
      sqlLine = "UPDATE clientes SET login = ?, AG_ID = ?, nombre= ?,telf= ?,DNI= ?,mail= ? WHERE CL_ID = ?";
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      ps.setInt(2,id);
      ps.setString(3,nombre);
      ps.setInt(4,telf);
      ps.setInt(5,DNI);
      ps.setString(6,mail);
      ps.setInt(7,idCliente);
    } else {
      sqlLine = "UPDATE clientes SET login = ?, AG_ID = ?, password = PASSWORD(?), nombre=?,telf=?,DNI=?,mail=? WHERE CL_ID = ?";
      ps = conexion.prepareStatement(sqlLine);
      ps.setString(1,login);
      ps.setInt(2,id);
      ps.setString(3,pass);
      ps.setString(4,nombre);
      ps.setInt(5,telf);
      ps.setInt(6,DNI);
      ps.setString(7,mail);
      ps.setInt(8,idCliente);
    }
    
    ps.executeUpdate();
    liberarConexion(conexion);
  }
  
  /**
   * Lista las polizas de los clientes de este agente
   * @return Coleccin Vector que contiene todos las polizas en forma de FichaPoliza
   */
  public Collection listarPolizas() throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    Collection c = new Vector();
    FichaPoliza f;
    ResultSet resultadoCL;
    ResultSet resultadoAux;
    String nombre;

    // Por cada uno de nuestros clientes, consultamos sus polizas
    Collection clientes = listarClientes();
    Iterator it = clientes.iterator();
    while (it.hasNext()) {
      FichaUsuario cliente = (FichaUsuario)it.next();
      nombre = cliente.getNombre();
      sqlLine = "SELECT * FROM polizas WHERE CL_ID = ?";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,cliente.getId());
      resultado = ps.executeQuery();
      while (resultado.next()) {
        // Creamos la fichaPoliza de este cliente
        f = new FichaPoliza(resultado.getInt("PO_ID"), resultado.getInt("CL_ID"),
                nombre, resultado.getString("matricula"), resultado.getString("modelo"),
                resultado.getInt("valor"), resultado.getInt("coste"));

        // Aadimos la Poliza al vector
        c.add(f);
      }
    }
    liberarConexion(conexion);
    return c;
  }
  
  /**
   * Da de alta una poliza asociada a un cliente de este agente 
   * @param idCliente Cliente al que se asocia esta poliza
   * @param matricula Matricula del vehculo asegurado
   * @param modelo Modelo del vehculo asegurado
   * @param valor Valor por el que se asegura el vehculo
   * @param coste Coste del seguro
   * @throws NoEncontrado El cliente no ha sido encontrado o no pertenece a este agente
   * @throws FaltanDatos si no ha rellenado todos los datos
   * @throws SQLException
   */
  public void altaPoliza(int idCliente, String matricula, String modelo, int valor, int coste) throws SQLException,FaltanDatos,NoEncontrado {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    String idPolizaCompleta;
    
    // Comprobamos que el cliente de la poliza pertenece a este agente
    sqlLine = "SELECT AG_ID FROM clientes WHERE CL_ID = ?"; 
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idCliente);
    resultado = ps.executeQuery();
    if (resultado.next()){
      // Existe el cliente pero no es suyo
      if (resultado.getInt("AG_ID") != id) {
        throw new NoEncontrado(); // El cliente al que quiere asociar la poliza no ha sido encontrado
      }
    } else {
      throw new NoEncontrado(); // El cliente al que quiere asociar la poliza no ha sido encontrado
    }
    
    // Buscamos cual debe ser el proximo identificador de poliza
    int idPoliza;
    sqlLine = "SELECT PO_ID FROM polizas ORDER BY PO_ID DESC"; 
    resultado = sentencia.executeQuery(sqlLine);
    if (resultado.next()){
      // Eliminamos el codigo de compaia y obtenemos el numero real de poliza
      idPoliza = Integer.parseInt((resultado.getString("PO_ID")).substring(3));
      idPoliza++; // Incrementamos
    } else {
      // No hay registros, es la primera poliza
      idPoliza = 1;
    }
    
    if ((matricula.equals("")) || (modelo.equals(""))) {
      throw new FaltanDatos();
    }
    // Obtenemos el id completo de la poliza juntando el id de la compaia y el id de la poliza 
    idPolizaCompleta = idCompania+""+idPoliza;
    sqlLine = "INSERT INTO polizas VALUES (?,?,?,?,?,?)";
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,idPolizaCompleta);
    ps.setInt(2,idCliente);
    ps.setString(3,matricula);
    ps.setString(4,modelo);
    ps.setInt(5,valor);
    ps.setInt(6,coste);
    ps.executeUpdate();
    liberarConexion(conexion);
  }
  
  /**
   * TODO: comprobar que el cliente de la poliza, es de este agente
   * Dar de baja la poliza indicada
   */
  public void bajaPoliza(int idPoliza) throws SQLException {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "DELETE FROM polizas WHERE PO_ID=?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idPoliza);
    ps.executeUpdate();
    liberarConexion(conexion);
  }
  
  /**
   * Obtiene la informacin de la poliza especificada
   * @param idPoliza Poliza de la que queremos obtener informacin
   * @return FichaUsuario del cliente que hemos solicitado
   */
  public FichaPoliza obtenerPoliza(int idPoliza) throws SQLException, NoEncontrado{
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    FichaPoliza f;
    
    sqlLine = "SELECT * FROM polizas WHERE PO_ID = ?"; 
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idPoliza);
    resultado = ps.executeQuery();
    if (resultado.next()) {
      // Obtenemos el nombre del cliente
      FichaUsuario cliente = obtenerCliente(resultado.getInt("CL_ID"));
      String nombre = cliente.getNombre();
      // Creamos la ficha de la poliza
      f = new FichaPoliza(resultado.getInt("PO_ID"), resultado.getInt("CL_ID"),
        nombre, resultado.getString("matricula"), resultado.getString("modelo"),
        resultado.getInt("valor"), resultado.getInt("coste"));
      liberarConexion(conexion);
      return f;
    } else {
      throw new NoEncontrado();
    }
  }
  
  /**
   * Modifica una poliza cuyo cliente es de este agente 
   * @param idPoliza Poliza que hay que modificar
   * @param matricula Matricula del vehculo asegurado
   * @param modelo Modelo del vehculo asegurado
   * @param valor Valor por el que se asegura el vehculo
   * @param coste Coste del seguro
   * @throws NoEncontrado La poliza no ha sido encontrada, el cliente de la poliza no ha sido encontrado o no pertenece a este agente
   * @throws FaltanDatos si no ha rellenado todos los datos
   * @throws SQLException
   */
  public void modificarPoliza(int idPoliza, String matricula, String modelo, int valor, int coste) throws SQLException,FaltanDatos,NoEncontrado {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    // Comprobamos que el cliente de la poliza pertenece a este agente
    sqlLine = "SELECT CL_ID FROM polizas WHERE PO_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idPoliza);
    resultado = ps.executeQuery();
    if (resultado.next()){
      sqlLine = "SELECT AG_ID FROM clientes WHERE CL_ID = ?"; 
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,resultado.getInt("CL_ID"));
      resultado = ps.executeQuery();
      if (resultado.next()){
        // Existe el cliente pero no es suyo
        if (resultado.getInt("AG_ID") != id) {
          throw new NoEncontrado(); // El cliente al que quiere asociar la poliza no ha sido encontrado
        }
      } else {
        throw new NoEncontrado(); // El cliente al que quiere asociar la poliza no ha sido encontrado
      }
    } else {
      throw new NoEncontrado(); // La poliza no existe
    }

    if ((matricula.equals("")) || (modelo.equals(""))) {
      throw new FaltanDatos();
    }
    sqlLine = "UPDATE polizas SET  matricula = ?, modelo = ?, valor=?, coste=? WHERE PO_ID = ?";      
    ps = conexion.prepareStatement(sqlLine);
    ps.setString(1,matricula);
    ps.setString(2,modelo);
    ps.setInt(3,valor);
    ps.setInt(4,coste);
    ps.setInt(5,idPoliza);
    ps.executeUpdate();
    liberarConexion(conexion);
  }
  
  /**
   * Lista los partes internos de los clientes de este agente, donde el cliente es causante
   * @param Estado Listar los partes que tiene el estado especificado
   * @return Coleccin Vector que contiene todos los partes en forma de FichaParte
   */
  public Collection listarPartesInternos(String Estado) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    Collection c = new Vector();
    FichaParteInterno f;
    boolean clienteCausante, yaInsertado;
    Iterator ite;

    // Por cada una de las polizas de nuestros clientes, consultamos sus partes
    Collection polizas = listarPolizas();
    Iterator it = polizas.iterator();
    while (it.hasNext()) {
      FichaPoliza poliza = (FichaPoliza)it.next();
      
      /////////////////
      // Quitamos esto, no se permite cambiar el estado de los partes creados por clientes nuestros (del agente)
      /////////////////
      /*sqlLine = "SELECT * FROM partes_internos WHERE (PO_ID = ? OR poliza_causante = ?) AND estado = ? ORDER BY fecha,hora DESC";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,poliza.getIdPoliza());
      ps.setInt(2,poliza.getIdPoliza());
      ps.setString(3,Estado);*/
      
      // Miramos los partes que han creado nuestros clientes
      sqlLine = "SELECT * FROM partes_internos WHERE poliza_causante = ? AND estado = ? ORDER BY fecha,hora DESC";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,poliza.getIdPoliza());
      ps.setString(2,Estado);
      resultado = ps.executeQuery();
      while (resultado.next()) {
        // Miramos si el causante es tb de nuestra compaia
        if ((resultado.getString("poliza_causante")).startsWith(idCompania)) {
          clienteCausante = true;
        } else {
          clienteCausante = false;
        }
        
        /////////////////
        // Quitamos esta comprobacin tambien, no se permite cambiar el estado de los partes creados por clientes nuestros (del agente)
        /////////////////
        // Debemos comprobar que el parte que estamos tratado no haya sido insertado ya
        // en la coleccion. Como las polizas de los clientes son comprobadas tanto para
        // causantes como victimas, puede haber situaciones que se traten partes por
        // duplicado.
        /*ite = c.iterator();
        yaInsertado = false;
        while (ite.hasNext()){
          f = (FichaParteInterno)ite.next();
          if (f.getId() == resultado.getInt("PI_ID")) {
            yaInsertado = true;
          }
        }
        
        if (! yaInsertado) {*/
          // Creamos la fichaParte
          f = new FichaParteInterno(resultado.getInt("PI_ID"), resultado.getInt("PO_ID"),
                  resultado.getInt("poliza_causante"), resultado.getString("estado"),
                  resultado.getString("descripcion"), resultado.getDate("fecha"),
                  resultado.getTime("hora"), resultado.getInt("indemnizacion"),
                  resultado.getString("descripcion_rechazo"), clienteCausante);
  
          // Aadimos la Poliza al vector
          c.add(f);
        //}
      }
    }
    liberarConexion(conexion);
    return c;
  }
  
  
  /**
   * Lista los partes internos de los clientes de este agente que sean victimas, 
   * los causantes no perteneceran a el mismo.
   * @param Estado Listar los partes que tiene el estado especificado
   * @return Coleccin Vector que contiene todos los partes en forma de FichaParte
   */
  public Collection listarPartesInternosNoModificables(String Estado) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    Collection c = new Vector();
    FichaParteInterno f;
    boolean clienteCausante, insertar;
    Iterator ite;

    // Por cada una de las polizas de nuestros clientes, consultamos sus partes
    Collection polizas = listarPolizas();
    Iterator it = polizas.iterator();
    while (it.hasNext()) {
      FichaPoliza poliza = (FichaPoliza)it.next();
      
      // Listamos los partes internos generados por asegurados nuestros
      sqlLine = "SELECT * FROM partes_internos WHERE PO_ID = ? AND estado = ? ORDER BY fecha,hora DESC";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,poliza.getIdPoliza());
      ps.setString(2,Estado);
      resultado = ps.executeQuery();
      
      while (resultado.next()) {
        // Miramos si el causante es tb de nuestra compaia
        if ((resultado.getString("poliza_causante")).startsWith(idCompania)) {
          clienteCausante = true;
          // Comprovamos si el causante es asegurado mio, si lo es no lo insertamos
          if (esClienteMio(resultado.getString("poliza_causante"))) {
            insertar = false;
          } else {
            insertar = true;
          }
        } else {
          clienteCausante = false;
          insertar = true;
        }
        
        if ( insertar) {
          // Creamos la fichaParte
          f = new FichaParteInterno(resultado.getInt("PI_ID"), resultado.getInt("PO_ID"),
                  resultado.getInt("poliza_causante"), resultado.getString("estado"),
                  resultado.getString("descripcion"), resultado.getDate("fecha"),
                  resultado.getTime("hora"), resultado.getInt("indemnizacion"),
                  resultado.getString("descripcion_rechazo"), clienteCausante);
  
          // Aadimos la Poliza al vector
          c.add(f);
        }
      }
    }
    liberarConexion(conexion);
    return c;
  }
      
      
  /**
   * Lista los partes de los clientes de este agente
   * @param Estado Listar los partes que tiene el estado especificado
   * @return Coleccin Vector que contiene todos los partes en forma de FichaParte
   */
  public Collection listarPartesExternos(String Estado) throws SQLException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    Collection c = new Vector();
    FichaParteExterno f;
    Iterator ite;

    // Por cada una de las polizas de nuestros clientes, consultamos sus partes
    Collection polizas = listarPolizas();
    Iterator it = polizas.iterator();
    while (it.hasNext()) {
      FichaPoliza poliza = (FichaPoliza)it.next();
      
      // Miramos los partes creados por usuarios externos
      sqlLine = "SELECT * FROM partes_externos WHERE PO_ID = ? AND estado = ? ORDER BY fecha,hora";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,poliza.getIdPoliza());
      ps.setString(2,Estado);
      resultado = ps.executeQuery();
      while (resultado.next()) {
        
        // Creamos la fichaParte
        f = new FichaParteExterno(resultado.getInt("PE_ID"), resultado.getString("id_parte_externo"), resultado.getInt("compania_acusadora"),
                resultado.getInt("PO_ID"), resultado.getString("estado"),
                resultado.getString("descripcion"), resultado.getDate("fecha"),
                resultado.getTime("hora"), resultado.getInt("indemnizacion"), 
                resultado.getString("descripcion_rechazo"));

        // Aadimos la Poliza al vector
        c.add(f);
      }
    }
    liberarConexion(conexion);
    return c;
  }
  
  /**
   * Comprueba que el parte interno especificado sea de este agente
   * @return true si el parte pertenece a este agente; false sino.
   */
  private boolean comprobarParteInterno(int idParte) throws SQLException, NoEncontrado{
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT PO_ID,poliza_causante FROM partes_internos WHERE PI_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idParte);
    resultado = ps.executeQuery();
    if (resultado.next()){
      int victima = resultado.getInt("PO_ID");
      int causante = resultado.getInt("poliza_causante");
      
      sqlLine = "SELECT CL_ID FROM polizas WHERE PO_ID = ?";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,victima);
      resultado = ps.executeQuery();
      if (resultado.next()){
        // Miramos si la victima es cliente nuestra y de ser asi comprovamos si pertenece
        // a este agente
        sqlLine = "SELECT AG_ID FROM clientes WHERE CL_ID = ?"; 
        ps = conexion.prepareStatement(sqlLine);
        ps.setInt(1,resultado.getInt("CL_ID"));
        resultado = ps.executeQuery();
        if (resultado.next()){
          // Existe el cliente pero no es suyo
          if (resultado.getInt("AG_ID") == id) {
            liberarConexion(conexion);
            return true;
          }
        }
      } 
      // Miramos si el causante es cliente nuestro y de ser asi comprovamos si pertenece
      // a este agente
      sqlLine = "SELECT CL_ID FROM polizas WHERE PO_ID = ?";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,causante);
      resultado = ps.executeQuery();
      if (resultado.next()){
        sqlLine = "SELECT AG_ID FROM clientes WHERE CL_ID = ?"; 
        ps = conexion.prepareStatement(sqlLine);
        ps.setInt(1,resultado.getInt("CL_ID"));
        resultado = ps.executeQuery();
        if (resultado.next()){
          // Existe el cliente pero no es suyo
          if (resultado.getInt("AG_ID") == id) {
            liberarConexion(conexion);
            return true;
          }
        }
      }
    } else {
      throw new NoEncontrado(); // El parte no existe
    }
    liberarConexion(conexion);
    return false;
  }
  
  
  /**
   * Comprueba que el parte externo especificado sea de este agente
   * @return true si el parte pertenece a este agente; false sino.
   */
  private boolean comprobarParteExterno(int idParte) throws SQLException, NoEncontrado{
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    sqlLine = "SELECT PO_ID FROM partes_externos WHERE PE_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idParte);
    resultado = ps.executeQuery();
    if (resultado.next()){
      int causante = resultado.getInt("PO_ID");

      // Miramos si el causante es cliente nuestro y de ser asi comprovamos si pertenece
      // a este agente
      // No hace falta mirar la victima, ya que al ser externa seguro que no es nuestra
      sqlLine = "SELECT CL_ID FROM polizas WHERE PO_ID = '" + causante + "'";
      resultado = sentencia.executeQuery(sqlLine);
      if (resultado.next()){
        sqlLine = "SELECT AG_ID FROM clientes WHERE CL_ID = ?"; 
        ps = conexion.prepareStatement(sqlLine);
        ps.setInt(1,resultado.getInt("CL_ID"));
        resultado = ps.executeQuery();
        if (resultado.next()){
          // Existe el cliente pero no es suyo
          if (resultado.getInt("AG_ID") == id) {
            liberarConexion(conexion);
            return true;
          }
        }
      }
    } else {
      throw new NoEncontrado(); // El parte no existe
    }
    liberarConexion(conexion);
    return false;
  }
  
  /**
   * Cambia el estado del parte interno especificado a RESOLVED y le asigna la indemnizacion
   * @param idParte Parte que se quiere resolver
   * @param indemnizacion Cantidad de la indemnizacion
   */
  public void resolverParteInterno(int idParte, int indemnizacion) throws SQLException, IOException, ProtocoloDesconocido, NoEncontrado, EstatusInvalido  {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    
    // Si el parte es de una poliza de un cliente de este agente
    if (comprobarParteInterno(idParte)) {
      sqlLine = "UPDATE partes_internos SET  estado = 'RESOLVED', indemnizacion = ? WHERE PI_ID = ?";      
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,indemnizacion);
      ps.setInt(2,idParte);
      ps.executeUpdate();
    } else {
      throw new NoEncontrado();
    }
    
    liberarConexion(conexion);
  }
  
  private void avisarParteInternoResuelto(String companiaID, String idParteExterno, int indemnizacion) throws IOException, ProtocoloDesconocido, NoEncontrado, EstatusInvalido, SQLException {
    Document doc = null;
    XMLgenerator xGen = new XMLgenerator();
    XMLextractor xExtract = new XMLextractor();
    
    // Obtenemos la informacin de la compaia
    EmpresaSeguros compania = obtenerCompania(Integer.parseInt(companiaID));
      
    Interlocutor locutor = new Interlocutor();
    doc = locutor.comunicar(compania.getProxy(), xGen.cambioEstadoResolvedRequest(idParteExterno, String.valueOf(indemnizacion)));
    
    String insuranceResponse = xExtract.filterErrorInsuranceResponse(doc);
    
    if (insuranceResponse.equals("CODE_NOT_FOUND")) {
      throw new NoEncontrado();
    } else if (insuranceResponse.equals("UNKNOWN_PROTOCOL")) {
      throw new ProtocoloDesconocido();
    } else if (insuranceResponse.equals("STATUS_NOT_VALID")) {
      throw new EstatusInvalido();
    }
  }
 
  
  /**
   * Cambia el estado del parte interno especificado a RESOLVED
   * @param idParte Parte que se quiere resolver
   * @param indemnizacion Cantidad de la indemnizacion
   */
  public void resolverNegativoParteInterno(int idParte) throws SQLException, NoEncontrado {
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    // Si el parte es de una poliza de un cliente de este agente
    if (comprobarParteInterno(idParte)) {
      sqlLine = "UPDATE partes_internos SET  estado = 'RESOLVED' WHERE PI_ID = ?";      
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,idParte);
      ps.executeUpdate();
    } else {
      throw new NoEncontrado();
    }
    liberarConexion(conexion);
  }
  
  /**
   * Cambia el estado del parte externo especificado a RESOLVED y le asigna la indemnizacion
   * @param idParte Parte que se quiere resolver negativamente
   * @param indemnizacion Cantidad de la indemnizacion
   */
  public void resolverParteExterno(int idParte, int indemnizacion) throws SQLException, NoEncontrado, ProtocoloDesconocido, EstatusInvalido, IOException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    // Avisar del cambio a otra compaia
    sqlLine = "SELECT * FROM partes_externos WHERE PE_ID = ?";      
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idParte);
    resultado = ps.executeQuery();
    
    if (resultado.next()) {
      avisarParteInternoResuelto(resultado.getString("compania_acusadora"), resultado.getString("id_parte_externo"), indemnizacion);
    }
    
    // Si el parte es de una poliza de un cliente de este agente
    if (comprobarParteExterno(idParte)) {
      sqlLine = "UPDATE partes_externos SET  estado = 'RESOLVED', indemnizacion = ? WHERE PE_ID = ?";      
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,indemnizacion);
      ps.setInt(2,idParte);
      ps.executeUpdate();
    } else {
      throw new NoEncontrado();
    }
    liberarConexion(conexion);
  }
  
  /**
   * Cambia el estado del parte externo especificado a RESOLVED
   * @param idParte Parte que se quiere resolver negativamente
   * @param indemnizacion Cantidad de la indemnizacion
   */
  public void resolverNegativoParteExterno(int idParte) throws SQLException, NoEncontrado, ProtocoloDesconocido, EstatusInvalido, IOException {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    // Avisar del cambio a otra compaia
    sqlLine = "SELECT * FROM partes_externos WHERE PE_ID = ?";      
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idParte);
    resultado = ps.executeQuery();
    
    if (resultado.next()) {
      // Resolver negativamente -> indemnizacion = 0
      avisarParteInternoResuelto(resultado.getString("compania_acusadora"), resultado.getString("id_parte_externo"), 0);
    }
    
    // Si el parte es de una poliza de un cliente de este agente
    if (comprobarParteExterno(idParte)) {
      sqlLine = "UPDATE partes_externos SET  estado = 'RESOLVED' WHERE PE_ID = ?";      
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,idParte);
      ps.executeUpdate();
    } else {
      throw new NoEncontrado();
    }
    liberarConexion(conexion);
  }
  
  /**
   * Obtiene la informacin del agente del cliente con la poliza indicada
   * @return FichaUsuario del agente que hemos solicitado
   */
  public ElementoAgente obtenerAgenteInterno(int idPoliza) throws SQLException, NoEncontrado{
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    ElementoAgente e;
    
    // Comprovamos que se trata de una poliza de un cliente de este agente
    sqlLine = "SELECT CL_ID FROM polizas WHERE PO_ID = ?"; 
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,idPoliza);
    resultado = ps.executeQuery();
    if (resultado.next()) {
      sqlLine = "SELECT AG_ID FROM clientes WHERE CL_ID = ?";
      ps = conexion.prepareStatement(sqlLine);
      ps.setInt(1,resultado.getInt("CL_ID"));
      resultado = ps.executeQuery();
      if (resultado.next()) {
        sqlLine = "SELECT * FROM agentes WHERE AG_ID = ?";
        ps = conexion.prepareStatement(sqlLine);
        ps.setInt(1,resultado.getInt("AG_ID"));
        resultado = ps.executeQuery();
        if (resultado.next()) {
          e = new ElementoAgente(resultado.getString("nombre"),
                resultado.getInt("telf"), resultado.getString("mail"));
          liberarConexion(conexion);
          return e;
        } else {
          throw new NoEncontrado();
        }
      } else {
          throw new NoEncontrado();
      }
     
    } else {
      throw new NoEncontrado();
    }
    
  }
  
    /**
   * Devuelve el id de la compaia acusadora de un parte externo
   * @param parte Parte externo
   * @return Integer con el id de la compaia
   */
  public int companiaAcusadoraParte(int parte) throws SQLException,NoEncontrado {
    ResultSet resultado;
    String sqlLine;
    Connection conexion = obtenerConexion();
    Statement sentencia = conexion.createStatement();
    PreparedStatement ps;
    
    
    // Miramos los partes creados por usuarios externos
    sqlLine = "SELECT * FROM partes_externos WHERE PE_ID = ?";
    ps = conexion.prepareStatement(sqlLine);
    ps.setInt(1,parte);
    resultado = ps.executeQuery();
    if (! resultado.next()) {
      throw new NoEncontrado();
    }
    liberarConexion(conexion);
    return resultado.getInt("PE_ID");
  }
  
  /**
   * Obtiene la informacin de una compaia registrada en Homer
   * @return FichaUsuario del agente que hemos solicitado
   */
  public EmpresaSeguros obtenerCompania(int compania) throws SQLException, ProtocoloDesconocido, NoEncontrado{
    Document doc = null;
    XMLgenerator xGen = new XMLgenerator();
    XMLextractor xExtract = new XMLextractor();
    //int compania = companiaAcusadoraParte(parte);
    try {
      Datos bd = new Datos();
      
      Interlocutor locutor = new Interlocutor();
      doc = locutor.comunicar(bd.getHomerURL(), xGen.getHomer(compania));
      
      String homerResponse = xExtract.filterErrorHomerResponse(doc);
      
      // Comprobamos las posibles respuestas de homer
      if (homerResponse.equals("NOT_FOUND")) {
        throw new NoEncontrado();
      } else if (homerResponse.equals("UNKNOWN_PROTOCOL")) {
        throw new ProtocoloDesconocido();
      } else {
        //try {
        return xExtract.traducirAEmpresa(doc);
        /*} catch (Exception e ){
        }
        return null;*/
      }
      
      
    } catch (IOException e) {
      throw new ProtocoloDesconocido();
    } catch (NamingException e){
      throw new NoEncontrado();
    }
  }
  
  /**
   * Obtiene la informacin del agente externo (de otra compaia) del cliente con la poliza indicada
   * @return FichaUsuario del agente que hemos solicitado
   */
  public ElementoAgente obtenerAgenteExterno(int poliza) throws SQLException, IOException, ProtocoloDesconocido, NoEncontrado{
    Document doc = null;
    XMLgenerator xGen = new XMLgenerator();
    XMLextractor xExtract = new XMLextractor();
    
    int polizaID = 0;
    int companiaID = 0;
    try {
      int size = (String.valueOf(poliza)).length();
      polizaID = Integer.parseInt((String.valueOf(poliza)).substring(3,size));
      companiaID = Integer.parseInt((String.valueOf(poliza)).substring(0,3));
    } catch (Exception e){
    }
    
    // Obtenemos la informacin de la compaia
    EmpresaSeguros compania = obtenerCompania(companiaID);
    try {
      
      Interlocutor locutor = new Interlocutor();
      doc = locutor.comunicar(compania.getProxy(), xGen.datosAgenteRequest(polizaID));
      
      String insuranceResponse = xExtract.filterErrorInsuranceResponse(doc);
      
      if (insuranceResponse.equals("CODE_NOT_FOUND")) {
        throw new NoEncontrado();
      } else if (insuranceResponse.equals("UNKNOWN_PROTOCOL")) {
        throw new ProtocoloDesconocido();
      } else {
        return xExtract.agenteContactData(doc);
      }
    
    } catch (IOException e) {
      throw new ProtocoloDesconocido();
    }
    
  }
 
  

}