package seguros;

import seguros.excepciones.*;

import java.io.*; 
import javax.servlet.*; 
import javax.servlet.http.*; 
import javax.naming.NamingException;

import java.sql.*;
import java.util.*;
import org.jdom.*;

/**
 * Servlet Controlador, crea los JavaBean necesarios, llama a sus metodos que
 * realizan modificaciones sobre la base de datos y redirige hacia la vista (JSP) adecuada.
 */
public class Controller extends HttpServlet {

  public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    String peticion = request.getServletPath();
    PrintWriter out = response.getWriter();
    HttpSession session = request.getSession(true);
    
    try {
      if (peticion.equals("/login.do")) {
        Datos bd = null;  // La inicializamos a null para que no nos diga despues
                          // del try-catch que la variable puede no estar inicializada
                          // pues si no va bien new Datos(), no pasaria mas all.

        bd = new Datos();
        String login = request.getParameter("login");
        String pass = request.getParameter("password");
      
        TipoUsuario tipo = bd.autorizar(login, pass);
        if (tipo.esAdmin()) {
          session.setAttribute("admin", bd.getAdmin(login));
          gotoPage("/admin.jsp", request, response);
        } else if (tipo.esAgente()) {
          session.setAttribute("agente", bd.getAgente(login));
          gotoPage("/agente.jsp", request, response);
        } else {
          session.setAttribute("cliente", bd.getCliente(login));
          gotoPage("/cliente.jsp", request, response);
        }
      } else if (peticion.equals("/logout.do")) {
          session.invalidate();
          gotoPage("/index.jsp", request, response);
      } else if (peticion.equals("/sinValidar.do")) {
          session.invalidate();
          gotoPage("/index.jsp", request, response);

      ////////////////////////////////////////////////////////////////////////////////
      ///////// ACCIONES DEL ADMIN
      ////////////////////////////////////////////////////////////////////////////////
      } else if (autentificadoAdmin(session)) { 
        // Dentro de este else estan todas las acciones en las cuales es necesario
        // que el usuario este validado como Admin
        if (peticion.equals("/menuAdmin.do")) {
          gotoPage("/admin.jsp", request, response);
          
          // Aadir nuestra compaia al servidor Homer
        } else if (peticion.equals("/addHomer.do")) {
          XMLgenerator x = new XMLgenerator();
          Interlocutor locutor = new Interlocutor();

          Admin adm = (Admin)session.getAttribute("admin");
          
          Document docResponse = locutor.comunicar(adm.getCompaniaHomer(), x.addHomer(adm.getCompaniaID(), adm.getCompaniaNombre(), adm.getCompaniaURL(), adm.getCompaniaIconURL(),  adm.getCompaniaProxyURL()));
          
          
          // Comprobar respuesta de Homer
          XMLextractor xExtract = new XMLextractor();
          String status=xExtract.filterErrorHomerResponse(docResponse);
          if (status.equals("OK")){
            response.sendRedirect("/seguros/listarHomer.do");
          } else if (status.equals("UNKNOWN_PROTOCOL")){
            response.sendRedirect("/seguros/error.jsp?error=ProtocoloDesconocido");
          } else if (status.equals("ALREADY_EXISTS")){
            response.sendRedirect("/seguros/error.jsp?error=HomerYaExiste");
          } else if (status.equals("NOT_FOUND")){
            response.sendRedirect("/seguros/error.jsp?error=HomerNoEncontrado");
          }
          
          // Quitar nuestra compaia del listado del servidor Homer
        } else if (peticion.equals("/removeHomer.do")) {
          XMLgenerator x = new XMLgenerator();
          Interlocutor locutor = new Interlocutor();

          Admin adm = (Admin)session.getAttribute("admin");
          
          Document docResponse = locutor.comunicar(adm.getCompaniaHomer(), x.removeHomer(adm.getCompaniaID()));
          
          // Comprobar respuesta de Homer
          XMLextractor xExtract = new XMLextractor();
          String status=xExtract.filterErrorHomerResponse(docResponse);
          if (status.equals("OK")){
            response.sendRedirect("/seguros/listarHomer.do");
          } else if (status.equals("UNKNOWN_PROTOCOL")){
            response.sendRedirect("/seguros/error.jsp?error=ProtocoloDesconocido");
          } else if (status.equals("ALREADY_EXISTS")){
            response.sendRedirect("/seguros/error.jsp?error=HomerYaExiste");
          } else if (status.equals("NOT_FOUND")){
            response.sendRedirect("/seguros/error.jsp?error=HomerNoEncontrado");
          }
          
          // Listar los servidores registrados en Homer
        } else if (peticion.equals("/listarHomer.do")) {
          XMLgenerator x = new XMLgenerator();
          Interlocutor locutor = new Interlocutor();
          
          Admin adm = (Admin)session.getAttribute("admin");
          
          Document docResponse = locutor.comunicar(adm.getCompaniaHomer(), x.listHomer()); 
          
          // Comprobar respuesta de Homer
          XMLextractor xExtract = new XMLextractor();
          String status=xExtract.filterErrorHomerResponse(docResponse);
          if (status.equals("OK")){
            request.setAttribute("empresas",xExtract.traducirAEmpresas(docResponse));
            gotoPage("/homer.jsp", request, response);
          } else if (status.equals("UNKNOWN_PROTOCOL")){
            response.sendRedirect("/seguros/error.jsp?error=ProtocoloDesconocido");
          } else if (status.equals("ALREADY_EXISTS")){
            response.sendRedirect("/seguros/error.jsp?error=HomerYaExiste");
          } else if (status.equals("NOT_FOUND")){
            response.sendRedirect("/seguros/error.jsp?error=HomerNoEncontrado");
          }
          
          
          // Formulario para cambiar la configuracion de la compaia
        } else if (peticion.equals("/companiaForm.do")) {
          Admin adm = (Admin)session.getAttribute("admin");
          
          Compania c = new Compania(adm.getCompaniaID(), adm.getCompaniaNombre(), adm.getCompaniaURL(), adm.getCompaniaIconURL(), adm.getCompaniaProxyURL(), adm.getCompaniaHomer(), adm.getCompaniaDTD());
          
          request.setAttribute("compania", c);
          gotoPage("/companiaForm.jsp", request, response);
          
          // Cambiar la configuracin de la compaia
        } else if (peticion.equals("/compania.do")) {
          // Obtenemos los datos
          String id = request.getParameter("id");
          String nombre = request.getParameter("nombre");
          String URL = request.getParameter("URL");
          String URLicon = request.getParameter("URLicon");
          String URLproxy = request.getParameter("URLproxy");
          String URLhomer = request.getParameter("URLhomer");
          String DTD = request.getParameter("DTD");
          
          if ((id.equals(""))||(nombre.equals(""))||(URL.equals(""))||(URLicon.equals(""))||(URLproxy.equals(""))||(URLhomer.equals(""))||(DTD.equals(""))) {
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            Admin adm = (Admin)session.getAttribute("admin");
            
            int iId = Integer.parseInt(id);
            
            adm.setCompaniaID(iId);
            adm.setCompaniaNombre(nombre);
            adm.setCompaniaURL(URL);
            adm.setCompaniaIconURL(URLicon);
            adm.setCompaniaProxyURL(URLproxy);
            adm.setCompaniaHomer(URLhomer);
            adm.setCompaniaDTD(DTD);
            response.sendRedirect("/seguros/menuAdmin.do");
          }
          
          // Formulario para cambiar el password
        } else if (peticion.equals("/passAdminForm.do")) {
          gotoPage("/passAdminForm.jsp", request, response);
          
          // Cambio de password
        } else if (peticion.equals("/passAdmin.do")) {
          String oldPass = request.getParameter("oldPass");
          String newPass1 = request.getParameter("newPass1");
          String newPass2 = request.getParameter("newPass2");
          
          if (newPass1.equals(newPass2)){
            Admin adm = (Admin)session.getAttribute("admin");
            // Comprobara que el pass viejo sea correcto, sino saltara una 
            // excepcion ViejoPassIncorrecto
            adm.setPassword(oldPass, newPass1);
            gotoPage("/menuAdmin.do", request, response);
          } else {
            response.sendRedirect("/seguros/error.jsp?error=PassNoCoincide");
          }
        } else {
          // Accion no disponible
          gotoPage("/error.jsp?error=NoAccion", request, response);
        }
        
      ////////////////////////////////////////////////////////////////////////////////
      ///////// ACCIONES DEL AGENTE
      ////////////////////////////////////////////////////////////////////////////////
      } else if (autentificadoAgente(session)) { 
        // Dentro de este else estan todas las acciones en las cuales es necesario
        // que el usuario este validado como Agente
        
        if (peticion.equals("/menuAgente.do")) {
          gotoPage("/agente.jsp", request, response);
          
        ////// Formulario para dar de alta un cliente
        } else if (peticion.equals("/altaClienteForm.do")) {
          gotoPage("/altaClienteForm.jsp", request, response);
          
        ////// Alta de un cliente
        } else if (peticion.equals("/altaCliente.do")) {
          // Obtenemos los datos
          String login = request.getParameter("login");
          String pass = request.getParameter("pass");
          String nombre = request.getParameter("nombre");
          String telf = request.getParameter("telf");
          String dni = request.getParameter("dni");
          String mail = request.getParameter("mail");
          
          if ((login.equals(""))||(pass.equals(""))||(nombre.equals(""))||(telf.equals(""))||(dni.equals(""))||(mail.equals(""))){
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            int iTelf = Integer.parseInt(telf);
            int iDNI = Integer.parseInt(dni);
  
            Agente ag = (Agente)session.getAttribute("agente");
            ag.altaCliente(login,pass,nombre,iTelf,iDNI,mail);
            response.sendRedirect("/seguros/listarClientes.do");
          }
          
        ////// Listar clientes
        } else if (peticion.equals("/listarClientes.do")) {
          Agente ag = (Agente)session.getAttribute("agente");
          request.setAttribute("clientes", ag.listarClientes());
          gotoPage("/listarClientes.jsp", request, response);
        
        ////// Dar de baja un cliente
        } else if (peticion.equals("/bajaCliente.do")) {
          String identificador = request.getParameter("ID");
          if (identificador.equals("")){
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            int id = Integer.parseInt(identificador);
            Agente ag = (Agente)session.getAttribute("agente");
            ag.bajaCliente(id);
            response.sendRedirect("/seguros/listarClientes.do");
          }
          
        ////// Mostrar el formulario para modificar un cliente
        } else if (peticion.equals("/modificarClienteForm.do")) {
          String identificador = request.getParameter("ID");
          if (identificador.equals("")){
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            int id = Integer.parseInt(identificador);
            Agente ag = (Agente)session.getAttribute("agente");
            request.setAttribute("cliente", ag.obtenerCliente(id));
            gotoPage("/modificarClienteForm.jsp", request, response);
          }
        
        ////// Modificar un cliente
        } else if (peticion.equals("/modificarCliente.do")) {
          // Obtenemos los datos
          String identificador = request.getParameter("ID");
          String login = request.getParameter("login");
          String pass = request.getParameter("pass");
          String nombre = request.getParameter("nombre");
          String telf = request.getParameter("telf");
          String dni = request.getParameter("dni");
          String mail = request.getParameter("mail");
          
          if ((login.equals(""))||(nombre.equals(""))||(telf.equals(""))||(dni.equals(""))||(mail.equals(""))){
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            int id = Integer.parseInt(identificador);
            int iTelf = Integer.parseInt(telf);
            int iDNI = Integer.parseInt(dni);
  
            Agente ag = (Agente)session.getAttribute("agente");
            ag.modificarCliente(id,login,pass,nombre,iTelf,iDNI,mail);
            response.sendRedirect("/seguros/listarClientes.do");
          }
        
        ////// Listar polizas
        } else if (peticion.equals("/listarPolizas.do")) {
          Agente ag = (Agente)session.getAttribute("agente");
          request.setAttribute("polizas", ag.listarPolizas());
          gotoPage("/listarPolizas.jsp", request, response);
          
        ////// Mostramos lista de clientes para que eliga a qual asociar la nueva poliza
        } else if (peticion.equals("/altaPolizaForm1.do")) {
          // Primera parte del alta de una poliza: la eleccin del cliente
          Agente ag = (Agente)session.getAttribute("agente");
          request.setAttribute("clientes", ag.listarClientes());
          gotoPage("/listarClientesPoliza.jsp", request, response);
          
        ////// Formulario de los datos para la nueva poliza
        } else if (peticion.equals("/altaPolizaForm2.do")) {
          // Segunda y ultima parte del alta de una poliza: rellenar los datos
          gotoPage("/altaPolizaForm.jsp", request, response);
          
        ////// Alta de la nueva poliza
        } else if (peticion.equals("/altaPoliza.do")) {
          // Obtenemos los datos
          String identificadorCliente = request.getParameter("ID");
          String matricula = request.getParameter("matricula");
          String modelo = request.getParameter("modelo");
          String valor = request.getParameter("valor");
          String coste = request.getParameter("coste");
          
          if ((identificadorCliente.equals(""))||(matricula.equals(""))||(modelo.equals(""))||(valor.equals(""))||(coste.equals(""))){
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            int idCliente = Integer.parseInt(identificadorCliente);
            int iValor = Integer.parseInt(valor);
            int iCoste = Integer.parseInt(coste);
  
            Agente ag = (Agente)session.getAttribute("agente");

            ag.altaPoliza(idCliente, matricula, modelo, iValor, iCoste);
            response.sendRedirect("/seguros/listarPolizas.do");
          }
          
        ////// Baja de una poliza
        } else if (peticion.equals("/bajaPoliza.do")) {
          String identificador = request.getParameter("ID");
          if (identificador.equals("")){
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            int id = Integer.parseInt(identificador);
            Agente ag = (Agente)session.getAttribute("agente");
            ag.bajaPoliza(id);
            response.sendRedirect("/seguros/listarPolizas.do");
          }
          
        ////// Formulario para modificar una poliza
        } else if (peticion.equals("/modificarPolizaForm.do")) {
          String identificador = request.getParameter("ID");
          if (identificador.equals("")){
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            int id = Integer.parseInt(identificador);
            Agente ag = (Agente)session.getAttribute("agente");
            request.setAttribute("poliza", ag.obtenerPoliza(id));
            gotoPage("/modificarPolizaForm.jsp", request, response);
          }
          
        ////// Modificar una poliza
        } else if (peticion.equals("/modificarPoliza.do")) {
          // Obtenemos los datos
          String identificador = request.getParameter("ID");
          String matricula = request.getParameter("matricula");
          String modelo = request.getParameter("modelo");
          String valor = request.getParameter("valor");
          String coste = request.getParameter("coste");
          
          if ((identificador.equals(""))||(matricula.equals(""))||(modelo.equals(""))||(valor.equals(""))||(coste.equals(""))){
            gotoPage("/error.jsp?error=FaltanDatos", request, response);
          } else {
            int idPoliza = Integer.parseInt(identificador);
            int iValor = Integer.parseInt(valor);
            int iCoste = Integer.parseInt(coste);
  
            Agente ag = (Agente)session.getAttribute("agente");
            ag.modificarPoliza(idPoliza, matricula, modelo, iValor, iCoste);
            response.sendRedirect("/seguros/listarPolizas.do");
          }
          
        ////// Listar partes
        } else if (peticion.equals("/listarPartes.do")) {
          Agente ag = (Agente)session.getAttribute("agente");
          
          // Partes internos donde el causante es cliente del agente y el causante puede ser o no
          Collection c = ag.listarPartesInternos("INITIAL");
          // Partes internos donde la victima es cliente del agente y el causante no * Los juntamos con los anteriores *
          c.addAll(ag.listarPartesInternosNoModificables("INITIAL"));
          request.setAttribute("partes_initial", c);
          request.setAttribute("partes_initial_ex", ag.listarPartesExternos("INITIAL"));
          
          // Partes internos donde el causante es cliente del agente y el causante puede ser o no
          c = ag.listarPartesInternos("VALIDATED");
          request.setAttribute("partes_validated", c);
          // Partes internos donde la victima es cliente del agente y el causante no. No los juntamos pq a estos no se debe permitir su modificacin
          c = ag.listarPartesInternosNoModificables("VALIDATED");
          request.setAttribute("partes_validated_no_modificables", c);
          request.setAttribute("partes_validated_ex", ag.listarPartesExternos("VALIDATED"));
          
          // Partes internos donde el causante es cliente del agente y el causante puede ser o no
          c = ag.listarPartesInternos("REJECTED");
          // Partes internos donde la victima es cliente del agente y el causante no * Los juntamos con los anteriores *
          c.addAll(ag.listarPartesInternosNoModificables("REJECTED"));
          request.setAttribute("partes_rejected", c);
          request.setAttribute("partes_rejected_ex", ag.listarPartesExternos("REJECTED"));
          
          // Partes internos donde el causante es cliente del agente y el causante puede ser o no
          c = ag.listarPartesInternos("RESOLVED");
          // Partes internos donde la victima es cliente del agente y el causante no * Los juntamos con los anteriores *
          c.addAll(ag.listarPartesInternosNoModificables("RESOLVED"));
          request.setAttribute("partes_resolved", c);
          request.setAttribute("partes_resolved_ex", ag.listarPartesExternos("RESOLVED"));
          
          gotoPage("/listarPartes.jsp", request, response);
          
        ////// Resolver un parte interno
        } else if (peticion.equals("/resolverParteInterno.do")) {
          // Obtenemos los datos
          String identificador = request.getParameter("ID");
          String indemnizacion = request.getParameter("indemnizacion");
          
          Agente ag = (Agente)session.getAttribute("agente");
          // Averiguamos si se trata de un parte interno o externo gracias a que
          // todos los partes internos tienen polizaVictima con codigo de la compaia
          int id = Integer.parseInt(identificador);
          int iIndemnizacion = Integer.parseInt(indemnizacion);
          
          ag.resolverParteInterno(id, iIndemnizacion);
          response.sendRedirect("/seguros/listarPartes.do");
          
        ////// Resolver un parte externo
        } else if (peticion.equals("/resolverParteExterno.do")) {
          // Obtenemos los datos
          String identificador = request.getParameter("ID");
          String indemnizacion = request.getParameter("indemnizacion");
          
          Agente ag = (Agente)session.getAttribute("agente");
          // Averiguamos si se trata de un parte interno o externo gracias a que
          // todos los partes internos tienen polizaVictima con codigo de la compaia
          int id = Integer.parseInt(identificador);
          int iIndemnizacion = Integer.parseInt(indemnizacion);
          
          ag.resolverParteExterno(id, iIndemnizacion);
          response.sendRedirect("/seguros/listarPartes.do");
        
        ////// Resolver un parte interno negativamente (Resuelve el parte con indemnizacion 0)
        } else if (peticion.equals("/resolverNegativoParteInterno.do")) {
          // Obtenemos los datos
          String identificador = request.getParameter("ID");
          
          Agente ag = (Agente)session.getAttribute("agente");
          // Averiguamos si se trata de un parte interno o externo gracias a que
          // todos los partes internos tienen polizaVictima con codigo de la compaia
          int id = Integer.parseInt(identificador);
          
          ag.resolverNegativoParteInterno(id);
          response.sendRedirect("/seguros/listarPartes.do");
          
        ////// Resolver un parte externo negativamente (Resuelve el parte con indemnizacion 0)
        } else if (peticion.equals("/resolverNegativoParteExterno.do")) {
          // Obtenemos los datos
          String identificador = request.getParameter("ID");
          
          Agente ag = (Agente)session.getAttribute("agente");
          // Averiguamos si se trata de un parte interno o externo gracias a que
          // todos los partes internos tienen polizaVictima con codigo de la compaia
          int id = Integer.parseInt(identificador);
          
          ag.resolverNegativoParteExterno(id);
          response.sendRedirect("/seguros/listarPartes.do");
        
        ////// Mostrar informacin de los agentes del parte
        } else if (peticion.equals("/infoAgente.do")) {
          // Obtenemos los datos
          String causante = request.getParameter("causante");
          String victima = request.getParameter("victima");
          Agente ag = (Agente)session.getAttribute("agente");
          // Averiguamos si desea consultar un agente interno o extero gracias a que
          // las polizas internas de agentes internos empiezan por el codigo de compaia
          int iCausante = Integer.parseInt(causante);
          int iVictima = Integer.parseInt(victima);;
          
          if (victima.startsWith(ag.getIdCompania())) {
            request.setAttribute("agenteVictima", ag.obtenerAgenteInterno(iVictima));
          } else {
            request.setAttribute("agenteVictima", ag.obtenerAgenteExterno(iVictima));
          }
          
          if (causante.startsWith(ag.getIdCompania())) {
            request.setAttribute("agenteCausante", ag.obtenerAgenteInterno(iCausante));
          } else {
            request.setAttribute("agenteCausante", ag.obtenerAgenteExterno(iCausante));
          }
          gotoPage("/mostrarAgentes.jsp", request, response);
          
          // Formulario para cambiar el password
        } else if (peticion.equals("/passAgenteForm.do")) {
          gotoPage("/passAgenteForm.jsp", request, response);
          
          // Cambio de password
        } else if (peticion.equals("/passAgente.do")) {
          String oldPass = request.getParameter("oldPass");
          String newPass1 = request.getParameter("newPass1");
          String newPass2 = request.getParameter("newPass2");
          
          if (newPass1.equals(newPass2)){
            Agente ag = (Agente)session.getAttribute("agente");
            // Comprobara que el pass viejo sea correcto, sino saltara una 
            // excepcion ViejoPassIncorrecto
            ag.setPassword(oldPass, newPass1);
            gotoPage("/menuAgente.do", request, response);
          } else {
            response.sendRedirect("/seguros/error.jsp?error=PassNoCoincide");
          }
        ////// Cualquier otra accin
        } else {
          // Accion no disponible
          gotoPage("/error.jsp?error=NoAccion", request, response);
        }
      
      ////////////////////////////////////////////////////////////////////////////////
      ///////// ACCIONES DEL CLIENTE
      ////////////////////////////////////////////////////////////////////////////////
      } else if (autentificadoCliente(session)) { 
        // Dentro de este else estan todas las acciones en las cuales es necesario
        // que el usuario este validado como Cliente
        if (peticion.equals("/menuCliente.do")) {
          gotoPage("/cliente.jsp", request, response);
          
          // Formulario para cambiar el password
        } else if (peticion.equals("/passClienteForm.do")) {
          gotoPage("/passClienteForm.jsp", request, response);
          
          // Cambio de password
        } else if (peticion.equals("/passCliente.do")) {
          String oldPass = request.getParameter("oldPass");
          String newPass1 = request.getParameter("newPass1");
          String newPass2 = request.getParameter("newPass2");
          
          if (newPass1.equals(newPass2)){
            Cliente cl = (Cliente)session.getAttribute("cliente");
            // Comprobara que el pass viejo sea correcto, sino saltara una 
            // excepcion ViejoPassIncorrecto
            cl.setPassword(oldPass, newPass1);
            gotoPage("/menuCliente.do", request, response);
          } else {
            response.sendRedirect("/seguros/error.jsp?error=PassNoCoincide");
          }
        } else {
          // Accion no disponible
          gotoPage("/error.jsp?error=NoAccion", request, response);
        }
      } else {
        // No esta intentando identificarse y no esta autentificado
        response.sendRedirect("/seguros/index.jsp");
      }

    } catch (EstatusInvalido e){
      gotoPage("/error.jsp?error=EstatusInvalido", request, response);
    } catch (ProtocoloDesconocido e){
      gotoPage("/error.jsp?error=ProtocoloDesconocido", request, response);
    } catch (NumberFormatException e){
      gotoPage("/error.jsp?error=CampoNumerico", request, response);
    } catch (LoginOcupado e){
      gotoPage("/error.jsp?error=LoginOcupado", request, response);
    } catch (ViejoPassIncorrecto e){
      gotoPage("/error.jsp?error=ViejoPassIncorrecto", request, response);
    } catch (FaltanDatos e){
      gotoPage("/error.jsp?error=FaltanDatos", request, response);
    } catch (NoValido e) {
      gotoPage("/index.jsp?error=NoValido", request, response);
    } catch (NoEncontrado e){
      gotoPage("/error.jsp?error=NoEncontrado", request, response);
    } catch (SQLException e) {
      request.setAttribute("excepcion", e.getMessage());
      gotoPage("/error.jsp?error=Excepcion", request, response);
    } catch (NamingException e) {
      request.setAttribute("excepcion", e.getMessage());
      gotoPage("/error.jsp?error=Excepcion", request, response);
    }
  }
  
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    doPost(request, response);
  }
  
  /**
   * Funcin que redirecciona a la pagina indicada, pasandole tb todo los parametros de entrada
   * @param address Direccin del servlet o jsp, siempre ha de empezar por /
   * @param request Request que se le pasar
   * @param response Response que se le pasar
   */
  private void gotoPage(String address, HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(address);
    dispatcher.forward(request, response);
  }
  
  /**
   * Comprueba si un usuario esta identificado como admin
   * @return true si el usuario ya esta identificado; false de lo contrario
   */
  private boolean autentificadoAdmin(HttpSession session){
    Admin admin = (Admin)session.getAttribute("admin");
    if (admin==null) {
      return false;
    } else {
      return true;
    }
  }
  
  /**
   * Comprueba si un usuario esta identificado como agente
   * @return true si el usuario ya esta identificado; false de lo contrario
   */
  private boolean autentificadoAgente(HttpSession session){
    Agente agente = (Agente)session.getAttribute("agente");
    
    if (agente==null) {
      return false;
    } else {
      return true;
    }
  }
  
  /**
   * Comprueba si un usuario esta identificado como cliente
   * @return true si el usuario ya esta identificado; false de lo contrario
   */
  private boolean autentificadoCliente(HttpSession session){
    Cliente cliente = (Cliente)session.getAttribute("cliente");
    
    if (cliente==null) {
      return false;
    } else {
      return true;
    }
  }
  
  
}