package seguros;

import seguros.excepciones.*;
import java.io.*;
import java.util.*;
import org.jdom.*; 
import java.sql.Date;
import java.sql.Time;

/**
 * Clase con metodos para extraer informacin de documentos XML
 * comunicacin entre compaias.
 */
public class XMLextractor {
  
  /**
   * Pasa un documento XML con el listado de las empresas registradas en Homer a una collection
   * @param mensaje Listado de las empresas en XML
   * @return Coleccin de EmpresaSeguros
   */
  public Collection traducirAEmpresas(Document mensaje) throws IOException, ProtocoloDesconocido{
    Collection result = new Vector();
    EmpresaSeguros emp = null;
    String proxyURL;
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element homer = OSDCMessage.getChild("HOMER");
      Element directoryResponse = homer.getChild("directoryResponse");
      List nodos = directoryResponse.getChildren();
      Iterator it = nodos.iterator();
      while (it.hasNext()){
        Element e = (Element)it.next();
        // La URL del proxy es opcional, por tanto comprobamos si la trae, de lo contrario sera vacio "".
        Element proxy = e.getChild("proxyURL");
        if (proxy==null){
          proxyURL = "";
        } else {
          proxyURL = proxy.getText();
        }
        //TODO: En lugar de 777 poner e.getChild("insuranceCode").getText()
        emp = new EmpresaSeguros("777", e.getChild("insuranceName").getText(), e.getChild("homeURL").getText(), e.getChild("iconURL").getText(),proxyURL);
        
        result.add(emp);
      }
      return result;
      
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }

  }
  
  /**
   * Pasa un documento XML con el resultado de 1 consulta de 1 empresa registrada en Homer a una EmpresaSeguros
   * @param mensaje Resultado de 1 consulta de 1 empresa en XML
   * @return EmpresaSeguros
   */
  public EmpresaSeguros traducirAEmpresa(Document mensaje) throws IOException, ProtocoloDesconocido{
    Collection result = new Vector();
    EmpresaSeguros emp = null;
    String proxyURL;
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element homer = OSDCMessage.getChild("HOMER");
      Element directoryResponse = homer.getChild("directoryResponse");
      Element e = directoryResponse.getChild("insuranceSite");
      // La URL del proxy es opcional, por tanto comprobamos si la trae, de lo contrario sera vacio "".
      Element proxy = e.getChild("proxyURL");
      if (proxy==null){
        proxyURL = "";
      } else {
        proxyURL = proxy.getText();
      }
      //TODO: En lugar de 777 poner e.getChild("insuranceCode").getText()
      emp = new EmpresaSeguros("777", e.getChild("insuranceName").getText(), e.getChild("homeURL").getText(), e.getChild("iconURL").getText(),proxyURL);
      return emp;
      
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }

  }
  
  
  /**
   * Devolvemos el estado del mensaje de respuesta a una peticion a Homer
   * @return String con el tipo de status: OK, UNKNOWN_PROTOCOL, ALREADY_EXISTS, NOT_FOUND
   */
  public String filterErrorHomerResponse(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element homer = OSDCMessage.getChild("HOMER");
      Element directoryResponse = homer.getChild("directoryResponse");
      return directoryResponse.getAttributeValue("status");
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos el estado del mensaje de respuesta a una peticion a una compaia
   * @return String con el tipo de status
   */
  public String filterErrorInsuranceResponse(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceResponse = insurance.getChild("insuranceResponse");
      return insuranceResponse.getAttributeValue("status");
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos el tipo de peticion
   * @return String con el tipo de peticion: notify, getStatusList, getDataContact o setStatus
   */
  public String filterRequest(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceRequest = insurance.getChild("insuranceRequest");
      List nodos = insuranceRequest.getChildren();
      Iterator it = nodos.iterator();
      if (it.hasNext()){
        Element e = (Element)it.next();
        return e.getName();
      } else {
        throw new ProtocoloDesconocido();
      }
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos un objeto FichaParteExterno a partir de un doc XML
   * @return String con el tipo de status
   */
  public FichaParteExterno notifyRequest (Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceRequest = insurance.getChild("insuranceRequest");
      Element notify = insuranceRequest.getChild("notify");
      String complaintCode = notify.getChild("complaintCode").getText();
      int insuranceCode = Integer.parseInt(notify.getChild("insuranceCode").getText());
      String description = notify.getChild("description").getText();
      int policy = Integer.parseInt(notify.getChild("policy").getText());
      
      Element timestamp = notify.getChild("timestamp");
      
      // Esta forma de construir Date y Time esta deprecated!
      java.sql.Date fecha = new java.sql.Date(Integer.parseInt(timestamp.getChild("year").getText()), Integer.parseInt(timestamp.getChild("month").getText()), Integer.parseInt(timestamp.getChild("day").getText()));
      
      java.sql.Time hora = new java.sql.Time(Integer.parseInt(timestamp.getChild("hh").getText()), Integer.parseInt(timestamp.getChild("mm").getText()), 0);
      
      return new FichaParteExterno(0, complaintCode, insuranceCode, policy, "INITIAL", description, fecha, hora, 0, "");

    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos la poliza que solicita que se liste sus partes
   * @return Integer con la poliza
   */
  public int policyStatusList(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceRequest = insurance.getChild("insuranceRequest");
      Element getStatusList = insuranceRequest.getChild("getStatusList");
      
      return Integer.parseInt(getStatusList.getChild("policy").getText());
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos la poliza que solicita que se liste sus partes
   * @return Integer con la poliza
   */
  public int parteContactData(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceRequest = insurance.getChild("insuranceRequest");
      Element getContactData = insuranceRequest.getChild("getContactData");
      
      return Integer.parseInt(getContactData.getChild("complaintCode").getText());
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos el nuevo estado que se solicita
   * @return String con el estado nuevo
   */
  public String setStatus(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceRequest = insurance.getChild("insuranceRequest");
      Element setStatus = insuranceRequest.getChild("setStatus");
      
      return setStatus.getAttributeValue("status");
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos ElementoResolved con la informacin del nuevo estado
   * @return ElementoResolved con la informacin del nuevo estado
   */
  public ElementoResolved setStatusResolved(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceRequest = insurance.getChild("insuranceRequest");
      Element setStatus = insuranceRequest.getChild("setStatus");
      
      return new ElementoResolved (Integer.parseInt(setStatus.getChild("complaintCode").getText()), Integer.parseInt(setStatus.getChild("amount").getText()));
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos el parte a cambiar a estado Validated
   * @return Integer que corresponde al identificador del parte
   */
  public int setStatusValidated(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceRequest = insurance.getChild("insuranceRequest");
      Element setStatus = insuranceRequest.getChild("setStatus");
      
      return Integer.parseInt(setStatus.getChild("complaintCode").getText());
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos ElementoResolved con la informacin del nuevo estado
   * @return ElementoResolved con la informacin del nuevo estado
   */
  public ElementoRejected setStatusRejected(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceRequest = insurance.getChild("insuranceRequest");
      Element setStatus = insuranceRequest.getChild("setStatus");
      
      return new ElementoRejected (Integer.parseInt(setStatus.getChild("complaintCode").getText()), setStatus.getChild("motivation").getText());
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
  
  /**
   * Devolvemos la informacin del agente
   * @return ElementoAgente con la info
   */
  public ElementoAgente agenteContactData(Document mensaje) throws ProtocoloDesconocido {
    try {
      Element OSDCMessage = mensaje.getRootElement();
      Element insurance = OSDCMessage.getChild("INSURANCE");
      Element insuranceResponse = insurance.getChild("insuranceResponse");
      Element contactData = insuranceResponse.getChild("contactData");
      
      String nombre = (contactData.getChild("name")).getText();
      int telephone = Integer.parseInt((contactData.getChild("telephone")).getText());
      String email = (contactData.getChild("email")).getText();
      
      return new ElementoAgente(nombre, telephone, email);
    } catch (Exception e){
      throw new ProtocoloDesconocido();
    }
  }
}
