package seguros;

import java.util.*;
import org.jdom.*; 

/**
 * Clase con metodos para generar consultas/respuestas en formato XML para la
 * comunicacin entre compaias.
 */
public class XMLgenerator {

  /**
   * Construye un documento XML con su correspondiente cabecera (dtd..)
   */
  private Document formarDocumento(Element e) {
    Document doc = null;
    try {
      Datos bd = new Datos();
      // La URL del DTD para validar el XML que generamos esta en la Base de datos
      doc = new Document(e);
      doc.setDocType(new DocType("OSDCMessage", bd.getDTD()));
    } catch (Exception ex) {
    }
    return doc;
  }
  
  /**
   * Metodo privado que genera una respuesta con el estado indicado
   * @param estado Se especificara el estado que se enviara en la respuesta
   * @return Document XML
   */
  private Document formarRespuesta (String estado) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceResponse = new Element("insuranceResponse");
    insuranceResponse.setAttribute("status", estado);
    INSURANCE.addContent(insuranceResponse);
    
    return formarDocumento(OSDCMessage);
  }
  /**
   * Genera un documento de solicitud de nuevoParte
   * @param complaint Codigo ID de la victima
   * @param insurance ID de la compaia
   * @param desc Descripcion del accidente
   * @param acusado Poliza del acusado
   * @param dia Dia del accidente
   * @param mes Mes del accidente
   * @param anyo Ao del accidente
   * @param hora Hora del accidente
   * @param minuto Minuto del accidente
   * @return Document XML
   */
  public Document nuevoParteRequest (String complaint, String insurance, String desc, String acusado, String dia, String mes, String anyo, String hora, String minuto) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceRequest = new Element("insuranceRequest");
    INSURANCE.addContent(insuranceRequest);
    
    Element notify = new Element("notify");
    insuranceRequest.addContent(notify);
    
    Element complaintCode = new Element("complaintCode");
    complaintCode.setText(complaint);
    notify.addContent(complaintCode);
    
    Element insuranceCode = new Element("insuranceCode");
    insuranceCode.setText(insurance);
    notify.addContent(insuranceCode);
    
    Element description = new Element(desc);
    description.setText("Se pegaron la ostia");
    notify.addContent(description);
    
    Element policy = new Element("policy");
    policy.setText(acusado);
    notify.addContent(policy);
    
    Element timestamp = new Element("timestamp");
    Element day = new Element(dia);
    day.setText("1");
    Element month = new Element(mes);
    month.setText("1");
    Element year = new Element(anyo);
    year.setText("1");
    Element hh = new Element(hora);
    hh.setText("1");
    Element mm = new Element(mes);
    mm.setText("1");
    timestamp.addContent(day);
    timestamp.addContent(month);
    timestamp.addContent(year);
    timestamp.addContent(hh);
    timestamp.addContent(mm);
    notify.addContent(timestamp);
    
    return formarDocumento(OSDCMessage);
  }
  
  
  /**
   * Respuesta a la peticion de nuevoParte
   * @return Document XML
   */
  public Document nuevoParteResponseOK () {
    return formarRespuesta("OK");
  }
  
  /**
   * Respuesta a la peticion de nuevoParte cuando no se ha encontrado la poliza especificada
   * @return Document XML
   */
  public Document nuevoParteResponseNotFound () {
    return formarRespuesta("POLICY_NOT_FOUND");
  }
  
  /**
   * Generara una peticion de listado de partes
   * @param asegurado Asegurado del que se quieren obtener los partes
   * @return Document XML
   */
  public Document listarPartesRequest (String asegurado) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceRequest = new Element("insuranceRequest");
    INSURANCE.addContent(insuranceRequest);
    
    Element getstatuslist = new Element("getStatusList");
    insuranceRequest.addContent(getstatuslist);
    
    Element policy = new Element("policy");
    policy.setText(asegurado);
    getstatuslist.addContent(policy);
    
    return formarDocumento(OSDCMessage);
  }
  
  /**
   * Genera una respuesta al listado de partes
   * @param lista Coleccion de ElementoParte en la cual se meteran todos los partes de respuesta (codigo + estado).
   * @return Document XML
   */
  public Document listarPartesResponse (Collection lista) {
    ElementoParte e;
    Element complaint;
    Element complaintCode;
    Element status;
    
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceResponse = new Element("insuranceResponse");
    insuranceResponse.setAttribute("status", "OK");
    INSURANCE.addContent(insuranceResponse);
    
    Iterator it = lista.iterator();
    while (it.hasNext()) {
      e = (ElementoParte)it.next();
      complaint = new Element("complaint");
      complaintCode = new Element("complaintCode");
      complaintCode.setText(e.getCode());
      complaint.addContent(complaintCode);
      
      status = new Element("status");
      status.setText(e.getStatus());
      complaint.addContent(status);
      
      insuranceResponse.addContent(complaint);
    }

    return formarDocumento(OSDCMessage);
  }
  
  
  /**
   * Genera una respuesta al listado de partes cuando no ha encontrado la poliza indicada
   * @return Document XML
   */
  public Document listarPartesResponseNotFound () {
    return formarRespuesta("POLICY_NOT_FOUND");
  }
  
  
  /**
   * Genera una respuesta cuando el protocolo utilizado es desconocido
   * @return Document XML
   */
  public Document protocoloDesconocido () {
    return formarRespuesta("UNKNOWN_PROTOCOL");
  }
  
  
  /**
   * Generara una peticion de listado de partes
   * @param asegurado Asegurado del que se quieren obtener los partes
   * @return Document XML
   */
  public Document datosAgenteRequest (int parte) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceRequest = new Element("insuranceRequest");
    INSURANCE.addContent(insuranceRequest);
    
    Element geContactData = new Element("getContactData");
    insuranceRequest.addContent(geContactData);
    
    Element complaintCode = new Element("complaintCode");
    complaintCode.setText(String.valueOf(parte));
    geContactData.addContent(complaintCode);
    
    return formarDocumento(OSDCMessage);
  }
  
  
  /**
   * Genera una respuesta al obtener datos agente
   * @param nombre Nombre del agente
   * @param telefono Telefono del agente
   * @param correo E-Mail del agente
   * @return Document XML
   */
  public Document datosAgenteResponse (ElementoAgente agente) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceResponse = new Element("insuranceResponse");
    insuranceResponse.setAttribute("status", "OK");
    INSURANCE.addContent(insuranceResponse);
    
    Element contactData = new Element("contactData");
    Element name = new Element("name");
    name.setText(agente.getNombre());
    contactData.addContent(name);
    
    Element telephone = new Element("telephone");
    telephone.setText(String.valueOf(agente.getTelefono()));
    contactData.addContent(telephone);
    
    Element email = new Element("email");
    email.setText(agente.getMail());
    contactData.addContent(email);
    
    insuranceResponse.addContent(contactData);

    return formarDocumento(OSDCMessage);
  }
  
  /**
   * Genera una respuesta al obtener datos agente cuando no lo ha encontrado
   * @return Document XML
   */
  public Document datosAgenteResponseNotFound () {
    return formarRespuesta("CODE_NOT_FOUND");
  }
  
  
  /**
   * Generara una peticion de cambio de estado de un parte a Resolved
   * @param parte Parte que cambiara de estado a Resolved
   * @param cantidad Cantidad de la indemnizacion
   * @return Document XML
   */
  public Document cambioEstadoResolvedRequest (String parte, String cantidad) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceRequest = new Element("insuranceRequest");
    INSURANCE.addContent(insuranceRequest);
    
    Element setStatus = new Element("setStatus");
    setStatus.setAttribute("status", "RESOLVED");
    insuranceRequest.addContent(setStatus);
    
    Element complaintCode = new Element("complaintCode");
    complaintCode.setText(parte);
    setStatus.addContent(complaintCode);
    
    Element amount = new Element("amount");
    amount.setText(cantidad);
    setStatus.addContent(amount);
    
    return formarDocumento(OSDCMessage);
  }
  
  /**
   * Generara una peticion de cambio de estado de un parte a Validated
   * @param parte Parte que cambiara de estado a Validated
   * @return Document XML
   */
  public Document cambioEstadoValidatedRequest (String parte) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceRequest = new Element("insuranceRequest");
    INSURANCE.addContent(insuranceRequest);
    
    Element setStatus = new Element("setStatus");
    setStatus.setAttribute("status", "VALIDATED");
    insuranceRequest.addContent(setStatus);
    
    Element complaintCode = new Element("complaintCode");
    complaintCode.setText(parte);
    setStatus.addContent(complaintCode);
    
    return formarDocumento(OSDCMessage);
  }
  
  /**
   * Generara una peticion de cambio de estado de un parte a Rejected
   * @param parte Parte que cambiara de estado a Rejected
   * @param motivo Motivo del rechazo del parte
   * @return Document XML
   */
  public Document cambioEstadoRejectedRequest (String parte, String motivo) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element INSURANCE = new Element("INSURANCE");
    OSDCMessage.addContent(INSURANCE);
    
    Element insuranceRequest = new Element("insuranceRequest");
    INSURANCE.addContent(insuranceRequest);
    
    Element setStatus = new Element("setStatus");
    setStatus.setAttribute("status", "REJECTED");
    insuranceRequest.addContent(setStatus);
    
    Element complaintCode = new Element("complaintCode");
    complaintCode.setText(parte);
    setStatus.addContent(complaintCode);
    
    Element motivation = new Element("motivation");
    motivation.setText(motivo);
    setStatus.addContent(motivation);
    
    return formarDocumento(OSDCMessage);
  }
  
  /**
   * Genera una respuesta al cambio de estado de un parte
   * @return Document XML
   */
  public Document cambioEstadoResponseOK () {
    return formarRespuesta("OK");
  }
  
  /**
   * Genera una respuesta al cambio de estado de un parte no encontrado
   * @return Document XML
   */
  public Document cambioEstadoResponseNotFound () {
    return formarRespuesta("CODE_NOT_FOUND");
  }
  
  /**
   * Genera una respuesta al cambio de estado incorrecto de un parte, e.g. pasar de rechazado a validado
   * @return Document XML
   */
  public Document cambioEstadoResponseInvalidStatus () {
    return formarRespuesta("STATUS_NOT_VALID");
  }
  
  
  /**
   * Generara una peticion de adicion al servidor Homer
   * @param codigo Codigo de la compaia
   * @param nombre Nombre de la compaia
   * @param URL URL de la web de la compaia
   * @param icono URL del logo de la compaia
   * @param proxy URL por donde comunicarse con la compaia
   * @return Document XML
   */
  public Document addHomer (String codigo, String nombre, String URL, String icono, String proxy) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element HOMER = new Element("HOMER");
    OSDCMessage.addContent(HOMER);
    
    Element directoryRequest = new Element("directoryRequest");
    directoryRequest.setAttribute("category", "insurance");
    HOMER.addContent(directoryRequest);
    
    Element add = new Element("add");
    directoryRequest.addContent(add);
    
    Element insuranceSite = new Element("insuranceSite");
    add.addContent(insuranceSite);
    
    Element insuranceCode = new Element("insuranceCode");
    insuranceCode.setText(codigo);
    insuranceSite.addContent(insuranceCode);
    
    Element insuranceName = new Element("insuranceName");
    insuranceName.setText(nombre);
    insuranceSite.addContent(insuranceName);
    
    Element homeURL = new Element("homeURL");
    homeURL.setText(URL);
    insuranceSite.addContent(homeURL);
    
    Element iconURL = new Element("iconURL");
    iconURL.setText(icono);
    insuranceSite.addContent(iconURL);
    
    Element proxyURL = new Element("proxyURL");
    proxyURL.setText(proxy);
    insuranceSite.addContent(proxyURL);
    
    return formarDocumento(OSDCMessage);
  }
  
  
  /**
   * Generara una peticion de borrado del servidor Homer
   * @param codigo Codigo de la compaia
   * @return Document XML
   */
  public Document removeHomer (String codigo) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element HOMER = new Element("HOMER");
    OSDCMessage.addContent(HOMER);
    
    Element directoryRequest = new Element("directoryRequest");
    directoryRequest.setAttribute("category", "insurance");
    HOMER.addContent(directoryRequest);
    
    Element remove = new Element("remove");
    directoryRequest.addContent(remove);
    
    Element insuranceCode = new Element("insuranceCode");
    insuranceCode.setText(codigo);
    remove.addContent(insuranceCode);
    
    return formarDocumento(OSDCMessage);
  }
  
  
  /**
   * Generara una peticion de datos de la compaia indicada
   * @param codigo Codigo de la compaia
   * @return Document XML
   */
  public Document getHomer (int codigo) {
    Element OSDCMessage = new Element("OSDCMessage");
    Element HOMER = new Element("HOMER");
    OSDCMessage.addContent(HOMER);
    
    Element directoryRequest = new Element("directoryRequest");
    directoryRequest.setAttribute("category", "insurance");
    HOMER.addContent(directoryRequest);
    
    Element get = new Element("get");
    directoryRequest.addContent(get);
    
    Element insuranceCode = new Element("insuranceCode");
    insuranceCode.setText(String.valueOf(codigo));
    get.addContent(insuranceCode);
    
    return formarDocumento(OSDCMessage);
  }
  
  /**
   * Generara una peticion de listado de compaias del servidor Homer
   * @return Document XML
   */
  public Document listHomer () {
    Element OSDCMessage = new Element("OSDCMessage");
    Element HOMER = new Element("HOMER");
    OSDCMessage.addContent(HOMER);
    
    Element directoryRequest = new Element("directoryRequest");
    directoryRequest.setAttribute("category", "insurance");
    HOMER.addContent(directoryRequest);
    
    Element list = new Element("list");
    directoryRequest.addContent(list);
    
    return formarDocumento(OSDCMessage);
  }
}
